import os
import json
import shutil
import zipfile
import pathlib
import requests
from pprint import pprint

#
# Copyright (c) 2021 Pharmacelera S.L.
# All rights reserved.
#
# Description: Pharmacelera API calls
#


class Calls:
    """Pharmacelera API calling class."""

    def __init__(self, params):
        """Initialize configured parameters"""
        self.__port = os.getenv("PORT") or params.get("PORT")
        self.__url = f"{params.get('URI')}:{self.__port}"
        self.__path = params.get("MODEL_NAME")

    def upload(self, files):
        """Upload binary and license inside container."""
        url = f"{self.__url}/uploadBinary"
        response = requests.request(
            "POST",
            url,
            files=files,
        )
        result = json.loads(response.text)
        return result

    def launch(self, endpoint, payload, files):
        """Launch an experiment with given data."""
        url = f"{self.__url}/{endpoint}"
        response = requests.request(
            "POST",
            url,
            data=payload,
            files=files,
        )
        result = json.loads(response.text)
        return result

    def get(self, id):
        """Get experiment status data."""
        url = f"{self.__url}?id={id}"
        response = requests.request(
            "GET",
            url,
        )
        result = json.loads(response.text)
        return result

    def download(self, id, folder, path=None):
        """Download a completed experiment."""
        url = f"{self.__url}/download?id={id}"
        zipName = folder + ".zip"
        with requests.request("GET", url, stream=True) as r:
            r.raise_for_status()
            zip_path = os.path.join("outputs", self.__path)
            pathlib.Path(os.path.join(path, zip_path)).mkdir(parents=True, exist_ok=True)
            with open(os.path.join(zip_path, zipName), "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
            if r.headers.get("content-type") == "application/zip":
               self.extractFiles(zip_path, zipName)

    def extractFiles(self, zip_path, zipName):
        """Extract sdf files from compressed files."""
        dirname = pathlib.Path().absolute()
        with zipfile.ZipFile(os.path.join(dirname, zip_path, zipName)) as zipped:
            for file in zipped.namelist():
                if (
                    file.endswith((".sdf", "mol2"))
                    and file.find("erroneous") < 0
                    and file.find("reference_molecules") < 0
                ):
                    filename = zipName.rsplit(".", 1)[0]
                    path = os.path.join(dirname, "tmp", filename)
                    zipped.extract(file, path)
                    ext = "." + file.split(".")[-1]
                    target = filename + ext
                    shutil.move(
                        os.path.join(path, file),
                        os.path.join(dirname, "tmp", target),
                    )
                    os.rmdir(path)

    def clearFiles(self):
        """Clear input and output files from container
        after finish and download the experiment.
        """
        url = f"{self.__url}/clearAll"
        response = requests.request("DELETE", url)
        result = json.loads(response.text)
        return result
